module TeXTable

include("./layout.jl");

export Table

ArrayOrMatrix = Union{Array{T,1}, Array{T,2}} where {T};

function ArrayConvert(s)
    if s == ","
        return " &"
    elseif s == "["
        return ""
    else
        return " \\\\\n"
    end
end

"""
Creates the body of the LaTeX table. Example: for the matrix [1 2 3; 4 5 6] it
creates:
1 & 2 & 3 \\
4 & 5 & 6 \\
"""
function body(x::ArrayOrMatrix)
    s = "";

    for r in eachrow(x)
        z = replace(string(r), r",|\[|\]" => ArrayConvert);
        s = string(s, TAB, TAB, z);
    end

    return s;
end

"""
Creates the alignment specifier for the columns.
"""
function alignment(align::AbstractString, columns::Integer)
    stripped = replace(align, r"\|" => "");
    if align == ""
        return repeat("l", columns);
    elseif length(stripped) == 1
        return replace(align, r"(l|c|r)" =>
                       repeat(match(r"(l|c|r)", align)[1],columns))
    elseif length(stripped) == columns
        return align;
    else
        error("Number of alignment specifiers must be equal to 1 or the number of columns.");
    end
end

"""
Joins the caption and label into one metadata block.
"""
function metadata(caption::AbstractString, label::AbstractString)
    if caption == "" && label == ""
        return ""
    elseif label == ""
        return string(TAB, "\\caption{", caption, "}\n");
    else
        return string(TAB, "\\caption{", caption, "\\label{", label, "}}\n");
    end
end

"""
Table(x::ArrayOrMatrix [, caption::AbstractString, label::AbstractString,
    align::AbstractString, digits::Int])

# Arguments
- x: The array or matrix to be converted to a LaTeX table.
- caption: Caption to show under the table.
- label: Label for LaTeX references.
- align: Alignment specifiers.
- digits: Number of digits to truncate the table to.

The alignment specifiers must either be equal to one or the number of columns
in the matrix. Alignment can be specified with sequences of 'l', 'c', or 'r',
with '|' in between for vertical lines for the table. For example, valid
alignment specifiers for a 3x3 matrix include 'l' -> "{lll}", '|r|| -> '{|rrr||},
'|ccc|' -> '{|ccc|}, and so on."""
function Table(x::ArrayOrMatrix; caption::AbstractString = "",
    label::AbstractString = "", align::AbstractString = "", digits::Int = -1)

    align = alignment(align, size(x, 2));

    if digits != -1
        x = trunc.(x, digits = digits);
    end

    meta = metadata(caption, label);

    return(string(top, "{", align, "}\n", hline, body(x), bottom, meta,
        tableend));
end

end # module
